@@ -2,6 +2,7 @@
  * connection.c
  *
  * Copyright (C) AB Strakt 2001, All rights reserved
+ * Copyright (C) Jean-Paul Calderone 2008, All rights reserved
  *
  * SSL Connection objects and methods.
  * See the file RATIONALE for a short explanation of why this module was written.
@@ -9,9 +10,6 @@
  * Reviewed 2001-07-23
  */
 #include <Python.h>
-#define SSL_MODULE
-#include <openssl/err.h>
-#include "ssl.h"
 
 #ifndef MS_WINDOWS
 #  include <sys/socket.h>
@@ -21,10 +19,13 @@
 #  endif
 #else
 #  include <winsock.h>
+#  include <wincrypt.h>
 #endif
 
-static char *CVSid = "@(#) $Id: connection.c,v 1.28 2004/08/06 10:21:56 martin Exp $";
+#define SSL_MODULE
+#include <openssl/err.h>
 
+#include "ssl.h"
 
 /**
  * If we are on UNIX, fine, just use PyErr_SetFromErrno. If we are on Windows,
@@ -756,6 +757,43 @@ ssl_Connection_set_app_data(ssl_ConnectionObj *self, PyObject *args)
     return Py_None;
 }
 
+static char ssl_Connection_get_shutdown_doc[] = "\n\
+Get shutdown state\n\
+\n\
+Arguments: self - The Connection object\n\
+           args - The Python argument tuple, should be empty\n\
+Returns:   The shutdown state, a bitmask of SENT_SHUTDOWN, RECEIVED_SHUTDOWN.\n\
+";
+static PyObject *
+ssl_Connection_get_shutdown(ssl_ConnectionObj *self, PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ":get_shutdown"))
+        return NULL;
+
+    return PyInt_FromLong((long)SSL_get_shutdown(self->ssl));
+}
+
+static char ssl_Connection_set_shutdown_doc[] = "\n\
+Set shutdown state\n\
+\n\
+Arguments: self - The Connection object\n\
+           args - The Python argument tuple, should be\n\
+             shutdown state - bitmask of SENT_SHUTDOWN, RECEIVED_SHUTDOWN.\n\
+Returns:   None\n\
+";
+static PyObject *
+ssl_Connection_set_shutdown(ssl_ConnectionObj *self, PyObject *args)
+{
+    int shutdown;
+
+    if (!PyArg_ParseTuple(args, "i:set_shutdown", &shutdown))
+        return NULL;
+
+    SSL_set_shutdown(self->ssl, shutdown);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static char ssl_Connection_state_string_doc[] = "\n\
 Get a verbose state description\n\
 \n\
@@ -888,6 +926,8 @@ static PyMethodDef ssl_Connection_methods[] =
     ADD_METHOD(makefile),
     ADD_METHOD(get_app_data),
     ADD_METHOD(set_app_data),
+    ADD_METHOD(get_shutdown),
+    ADD_METHOD(set_shutdown),
     ADD_METHOD(state_string),
     ADD_METHOD(sock_shutdown),
     ADD_METHOD(get_peer_certificate),
